Wire

Struct Wire 

pub struct Wire<T>
where T: ?Sized,
{ /* private fields */ }
Wraps and transfers complex objects over FFI.

The backing storage uses a (ptr, size) representation that can safely cross FFI boundaries. See the module documentation for more details and examples.

Implementations

Source

impl<T: TypeInfo + WireIO> Wire<T>

pub fn try_from(value: T) -> Result<Self, SerializationError>

Serialize value into a new owned Wire.

Errors

Returns SerializationError if value cannot be serialized into the buffer.

pub fn from(value: T) -> Self

Serialize value into a new owned Wire.

Panics

Panics at compile time if T::WIRE_SAFE is false.

pub fn try_unwire(&mut self) -> Result<T, SerializationError>

Deserialize the value from this Wire’s buffer.

Errors

Returns SerializationError if the buffer contents cannot be deserialized into T (e.g., truncated buffer, malformed data).

pub fn unwire(&mut self) -> T

Deserialize the value from this Wire’s buffer.

Panics

Panics at compile time if T::WIRE_SAFE is false.