pub struct Wire<T>where
T: ?Sized,{ /* private fields */ }Expand description
Wraps and transfers complex objects over FFI.
The backing storage uses a (ptr, size) representation that can safely cross FFI boundaries. See the module documentation for more details and examples.
Implementations
Sourceimpl<T: TypeInfo + WireIO> Wire<T>
impl<T: TypeInfo + WireIO> Wire<T>
Sourcepub fn try_from(value: T) -> Result<Self, SerializationError>
pub fn try_from(value: T) -> Result<Self, SerializationError>
Serialize value into a new owned Wire.
Errors
Returns SerializationError if value cannot be serialized into the buffer.
Sourcepub fn try_unwire(&mut self) -> Result<T, SerializationError>
pub fn try_unwire(&mut self) -> Result<T, SerializationError>
Deserialize the value from this Wire’s buffer.
Errors
Returns SerializationError if the buffer contents cannot be deserialized
into T (e.g., truncated buffer, malformed data).